class NUMERIC_TEST

inherit
	AEL_PRINTF

create
	make

feature {NONE} -- Initialization

	make
			-- Run application.
		do
			create tests_passed.make
			create tests_failed.make
			show_expected_on_pass := True

			exec_fraction_test1
			exec_fraction_test2
			exec_fraction_test3

			exec_numeric_test1
			exec_numeric_test2

			exec_bit_test1

			report_results
		end

--|========================================================================
feature -- Tests
--|========================================================================

	exec_fraction_test1 is
		local
			v1: REAL_64
			tf: AEL_FRACTION
			test_name, ok_out, t_out: STRING
			desc: STRING
		do
			test_name := "Fraction test 1"
			desc := "Convert real 64 into a fraction"
			v1 := 123.625
			tf := fr.float_to_fraction (v1)
			t_out := tf.out
			ok_out := "989/8"
			record_result (v1, t_out, ok_out, test_name, 1, desc)
			t_out := tf.out_long
			ok_out := "123 5/8"
			record_result (v1, t_out, ok_out, test_name, 2, desc)
		end

	--|--------------------------------------------------------------

	exec_fraction_test2 is
		local
			v1: REAL_64
			tf: AEL_USBLD_FRACTION
			test_name, ok_out, t_out: STRING
			desc: STRING
		do
			test_name := "Fraction test 2"
			desc := "Create US building convention fraction from real 64"
			v1 := 123.635
			create tf.make_with_value (v1)
			t_out := tf.out
			ok_out := "989/8"
			record_result (v1, t_out, ok_out, test_name, 1, desc)
			t_out := tf.out_long
			ok_out := "123 5/8"
			record_result (v1, t_out, ok_out, test_name, 2, desc)
		end

	--|--------------------------------------------------------------

	exec_fraction_test3 is
		local
			v1: REAL_64
			test_name, ok_out, t_out: STRING
			desc: STRING
			tp: TUPLE [INTEGER, INTEGER, INTEGER, INTEGER]
		do
			test_name := "Fraction test 3"
			desc := "Convert real 64 into a fraction, using fuzzy"
			v1 := 123.625
			tp := fr.fuzzy_float_to_fractional_parts (
				v1, 3, 0, << 2, 3, 4, 5, 6, 7, 8, 9, 10 >>)
			t_out := aprintf (
				"%%d,%%d,%%d,%%d",
				<< tp.integer_item (1),
				tp.integer_item (2),
				tp.integer_item (3),
				tp.integer_item (4)>>)
			ok_out := "123,5,8,3"
			record_result (v1, t_out, ok_out, test_name, 1, desc)
		end

	--|--------------------------------------------------------------

	exec_numeric_test1 is
		local
			v1: REAL_64
			ti: INTEGER
			test_name, ok_out, t_out: STRING
			desc: STRING
		do
			test_name := "Numeric test 1"
			desc := "Real 64 rounded up to nearest mod 4, 8, 32, 64, 117"
			v1 := 123.625
			ti := nr.real_rounded_to_next_even_mod (v1, 4, True)
			t_out := ti.out
			ok_out := "124"
			record_result (v1, t_out, ok_out, test_name, 1, desc)

			ti := nr.real_rounded_to_next_even_mod (v1, 8, True)
			t_out := ti.out
			ok_out := "128"
			record_result (v1, t_out, ok_out, test_name, 2, desc)

			ti := nr.real_rounded_to_next_even_mod (v1, 32, True)
			t_out := ti.out
			ok_out := "128"
			record_result (v1, t_out, ok_out, test_name, 3, desc)

			ti := nr.real_rounded_to_next_even_mod (v1, 64, True)
			t_out := ti.out
			ok_out := "128"
			record_result (v1, t_out, ok_out, test_name, 4, desc)

			ti := nr.real_rounded_to_next_even_mod (v1, 117, True)
			t_out := ti.out
			ok_out := "234"
			record_result (v1, t_out, ok_out, test_name, 5, desc)

			ti := nr.real_rounded_to_next_even_mod (0, 8, True)
			t_out := ti.out
			ok_out := "0"
			record_result (0, t_out, ok_out, test_name, 6, desc)
		end

	--|--------------------------------------------------------------

	exec_numeric_test2 is
		local
			v1: INTEGER_32
			v2: INTEGER_64
			t32: INTEGER_32
			t64: INTEGER_64
			test_name, ok_out, t_out: STRING
			desc: STRING
		do
			test_name := "Numeric test 2"
			desc := "Integer rounded up to nearest multiple of 8, 10"
			v1 := 123
			v2 := 123

			t32 := nr.even_8_multiple (v1)
			t_out := t32.out
			ok_out := "128"
			record_result (v1, t_out, ok_out, test_name, 1, desc)

			t64 := nr.even_8_multiple_64 (v2)
			t_out := t64.out
			ok_out := "128"
			record_result (v1, t_out, ok_out, test_name, 2, desc)

			t32 := nr.even_10_multiple (v1)
			t_out := t32.out
			ok_out := "130"
			record_result (v1, t_out, ok_out, test_name, 3, desc)

			t64 := nr.even_10_multiple_64 (v2)
			t_out := t64.out
			ok_out := "130"
			record_result (v1, t_out, ok_out, test_name, 4, desc)

			t32 := nr.even_8_multiple (0)
			t_out := t32.out
			ok_out := "8"
			record_result (0, t_out, ok_out, test_name, 5, desc)

			t32 := nr.even_10_multiple (0)
			t_out := t32.out
			ok_out := "10"
			record_result (0, t_out, ok_out, test_name, 6, desc)
		end

	--|--------------------------------------------------------------

	exec_bit_test1 is
		local
			v1: NATURAL_8
			tv: INTEGER
			test_name, ok_out, t_out: STRING
			desc: STRING
		do
			test_name := "Bit test 1"
			desc := "Natural 8 with some bits sets"
			v1 := 0x4c

			tv := nr.highest_bit_set (v1)
			t_out := tv.out
			ok_out := "6"
			record_result (v1, t_out, ok_out, test_name, 1, desc)

			tv := nr.highest_bit_unset (v1)
			t_out := tv.out
			ok_out := "7"
			record_result (v1, t_out, ok_out, test_name, 2, desc)

			tv := nr.lowest_bit_set (v1)
			t_out := tv.out
			ok_out := "2"
			record_result (v1, t_out, ok_out, test_name, 3, desc)

			tv := nr.lowest_bit_unset (v1)
			t_out := tv.out
			ok_out := "0"
			record_result (v1, t_out, ok_out, test_name, 4, desc)

			tv := nr.byte_complement (v1)
			t_out := tv.out
			ok_out := "179"
			record_result (v1, t_out, ok_out, test_name, 5, desc)
		end

--|========================================================================
feature {NONE} -- Recording and reporting
--|========================================================================

	record_result (
		v: REAL_64; t_out, ok_out, nm: STRING; id: INTEGER; desc: STRING) is
		local
			ts: STRING
		do
			ts := aprintf ("%%s.%%d - %%s", <<nm, id, desc>>)
			if t_out.is_equal (ok_out) then
				if show_expected_on_pass then
					ts := aprintf (
						"%%s%N    Value    [%%10.5f]%N    Expected [%%s]",
						<<ts, v, ok_out>>)
				end
				tests_passed.extend (ts)
			else
				ts := aprintf (
					"%%s%N    Value    [%%10.5f]%N    Expected [%%s]%N    Actual   [%%s]",
					<<ts, v, ok_out, t_out>>)
				tests_failed.extend (ts)
			end
		end

	--|--------------------------------------------------------------

	report_results is
		do
			print ("Tests Failed%N")
			print_list (tests_failed)
			print ("Tests Passed%N")
			print_list (tests_passed)
			printf (
				"Totals:%N  %%-8d Passed%N  %%-8d Failed%N",
				[tests_passed.count, tests_failed.count])
		end

	--|--------------------------------------------------------------

	print_list (tl: like tests_passed) is
		do
			if tl.is_empty then
				print ("  NONE%N")
			else
				from tl.start
				until tl.exhausted
				loop
					printf ("  %%s%N", tl.item)
					tl.forth
				end
			end
		end

	--|--------------------------------------------------------------

	tests_passed: LINKED_LIST [STRING]
	tests_failed: LINKED_LIST [STRING]

	show_expected_on_pass: BOOLEAN

--|========================================================================
feature {NONE} -- Numeric routines
--|========================================================================

	fr: AEL_FRACTION_ROUTINES is
		once
			create Result
		end

	nr: AEL_NUMERIC_ROUTINES is
		once
			create Result
		end

end
